package com.nisum.paymentvendorsimulator.creditcard.controllers;

import com.nisum.paymentvendorsimulator.creditcard.domain.dto.CreditCardRequest;
import com.nisum.paymentvendorsimulator.creditcard.domain.dto.CreditCardResponse;
import com.nisum.paymentvendorsimulator.creditcard.services.CreditCardService;
import com.nisum.paymentvendorsimulator.exceptions.BaseController;
import com.nisum.paymentvendorsimulator.exceptions.ErrorModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.*;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Valid;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/cc")
@CrossOrigin(origins = "*")
public class CreditCardController{

  private static final Logger logger = LoggerFactory.getLogger(CreditCardController.class);

  @Autowired
  CreditCardService ccService;

  @PostMapping(value = "/authorize", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
  public CreditCardResponse authorize(@Valid @RequestBody CreditCardRequest ccRequest) {
    logger.info("this is CC controller");
    return ccService.authorize(ccRequest);
  }

  @ExceptionHandler(MethodArgumentNotValidException.class)
  @ResponseStatus(HttpStatus.BAD_REQUEST)
  public ResponseEntity<ErrorModel> handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
    ErrorModel errorModel = new ErrorModel();

    List<String> errors = e.getAllErrors().stream()
            .map(error -> error.getDefaultMessage())
            .collect(Collectors.toList());

    errorModel.setDeveloperMessage("Bad Request");
    errorModel.setUserMessage("Errors: " + errors.toString());
    errorModel.setErrorCode(HttpStatus.BAD_REQUEST.value());
    errorModel.setMoreInfo("/swagger-ui/index.html#/credit-card-controller/authorizeUsingPOST_1");

    return new ResponseEntity<>(errorModel, HttpStatus.BAD_REQUEST);
  }

}
